# -*- coding: euc-kr -*-
# -*- test-case-name: lupy.test.test_document -*-
"""Documents and Fields"""
# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

class Document(object):
    """Documents ε̰ ġ ⺻ ̴.
    
    Documents fields ̴.  ʵ name   ִ.
    ʵ Documents Բ  ǰ ׷쿡 Document ˻ Ƚ ȯ ȴ.
    ׷ Ϲ  Document װ    ʵ带 Ѵ.
    """
    """Documents are the unit of indexing and search.

    A Document is a set of fields.  Each field has a name and a textual
    value.  A field may be stored with the document, in which case it is
    returned with search hits on the document.  Thus each document should
    typically contain stored fields which uniquely identify it.
    """

    def __init__(self):
        """ʵ鸦  ڷᱸ """
        self._fields = {} # ̸ Űǰ  field ü ȴ.
        self.fieldNames = [] #̸ ϴ  ׷ ̰ Python   ʿ䰡 .
        #self._fields.keys() Ͱ Ʈ ȯ  ִ.
    
    def add(self, field):
        """Document filed ü Ѵ.  ̸ ũؾѴ.
        ߺ˻縦 ϰ  Append
        """
        name = field.name()
        self._fields[name] = field
        if name not in self.fieldNames:
            self.fieldNames.append(name)
    
    def getField(self, name):
        """̸ õ ʵ尴ü ȯѴ."""
        return self._fields.get(name, None)

    def get(self, name):
        """ش ʵ Stringü ȯѴ."""
        field = self.getField(name)
        if field is not None:
            return field.stringValue()
        else:
            return None

    def fields(self):
        """ ʵ ü Ʈ Ѵ."""
        return [self._fields[name] for name in self.fieldNames]

    def removeField(name):
        """õ ʵ带 Ѵ. ׷  ι ؾϳ????"""
        self.fieldNames.remove(name)
        del self._fields[name]

    def __repr__(self):
        """ ʵ̸ """
        return '<Document[%s]>' % ("|".join(self.fieldNames),)


class Field(object):
    """ϳ Field Document  κ̴.
    
     ʵ ΰ Ʈ ȴ. name value̴.
    Values Ƹ ο ؽƮ ̰, String̳ Ϸ   ̴, Ǵ  ٸ μ
     ʿ ּ Ű  ִ.. ̷ Ű ¥, url  ǥϴµ ϼ ִ.
    document Ʈ ϱ, Fields  ε ȴ.

    Keyword : м м  ˻ ϸ  ȴ.(¥, url ϼ ִ.)
    UnIndexed : м м ʰ ˻ Ұϴ.  ๮ ۼ    ִ.
    UnStored : UnIndexed ݴ м мϰ ˻  ο   ʴ´.
    Text : м мǸ εȴ.

    """
    """A field is a section of a Document.

    Each field has two parts, a name and a value.  Values may be free
    text, provided as a string or as a file, or they may be atomic
    keywords, which are not further processed.  Such keywords may be used
    to represent dates, urls, etc.  Fields are optionally stored in the
    index, so that they may be returned with hits on the document.
    """

    def __init__(self, name, string, store=False, index=True, token=True):
        self.nom = name
        self.stringVal = string
        self.readerVal = None
        self.isStored = store
        self.isIndexed = index
        self.isTokenized = token        

    def __repr__(self):
        if self.isStored and self.isIndexed and not self.isTokenized:
            return '<Keyword<' + self.nom + ':' + self.stringVal + '>>'
        elif self.isStored and not self.isIndexed and not self.isTokenized:
            return '<Unindexed<' + self.nom + ':' + self.stringVal + '>>'
        elif self.isStored and self.isIndexed and self.isTokenized and self.stringVal is not None:
            return '<Text<' + self.nom + ':' + self.stringVal + '>>'
        elif not self.isStored and self.isIndexed and self.isTokenized and self.readerVal is not None: # κ 
            return '<Text<' + self.nom + ':' + self.readerVal + '>>'        	
        elif not self.isStored and self.isIndexed and self.isTokenized and self.stringVal is not None:
            return '<UnStored<' + self.nom + ':' + self.stringVal + '>>'        	
        else:
            return '<Field<???>'

    def name(self):
        return self.nom

    def stringValue(self):
        return self.stringVal

    def readerValue(self):
        return self.readerVal


def Keyword(name, value):
    """м м  ˻ ϸ  ȴ.(¥, url ϼ ִ.)"""
    return Field(name, value, True, True, False)


def Text(name, strOrFile, store=True):
    """м мǸ εȴ."""
    if isinstance(strOrFile, (str, unicode)):
        res = Field(name, strOrFile, store, True, True)
    else:
        res = Field(name, None) #Ʈ   ȵǰ ε ūȭ ǰ
        res.readerVal = strOrFile
        res.stringVal = None
    return res


def UnIndexed(name, value):
    """м м ʰ ˻ Ұϴ.  ๮ ۼ    ִ."""
    return Field(name, value, True, False, False)


def UnStored(name, value):
    """UnIndexed ݴ м мϰ ˻  ο   ʴ´."""
    return Field(name, value, False, True, True)


if __name__=="__main__":
    doc = Document()

